package GUI;

import Game.Card;
import Game.Card.Suit;
import java.awt.Image;
import java.awt.Toolkit;
import java.io.File;

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 *
 * @author A.M. Bosch, S080431
 */
public class CardImage {

    //constants

    // Instance
    private static CardImage cardImage = new CardImage();

    // Card image width and height
    final public static int WIDTH = 120;
    final public static int HEIGHT = 185;
    
    // Values for indenting and spacing when drawing images
    final public static int YINDENT = (int)(HEIGHT*41.0/215.0);
    final public static int XSPACING = (int)(WIDTH*50.0/150.0);
    final public static int YSPACING = (int)(HEIGHT*41.0/215.0);

    //variables
    private String imagesPath;

    // Image objects
    private Image cardImages[][] = new Image[4][13];
    private Image suitImages[] = new Image[4];
    private Image backImage;

    //constructor
    private CardImage() {
        loadCards();
    }

    //methods
    public static CardImage getCardImage() {
        return cardImage;
    }

    /*
     * Loads all the Image objects with handles to the card images.
     * Loads the imagesPath of the local project to find the path to these images.
     */
    private void loadCards() {
        final String cardRankStr = "23456789tjqka";
        final String cardSuitStr = "cdsh";
        final Toolkit toolkit = Toolkit.getDefaultToolkit();

        // Get cross-platform filepath seperator and project root
        final String pathSep = System.getProperty("file.separator");
        final File projectRoot = new File("");

        // The path where the images are stored is <ProjectRoot>/img/CardImages
        imagesPath = projectRoot.getAbsolutePath() + pathSep + "img" +
                pathSep + "CardImages" + pathSep;

        // Buffers all card images in 2D array
        for (int i = 0; i < cardSuitStr.length(); i++) {
            for (int j = 0; j < cardRankStr.length(); j++) {
                cardImages[i][j] = toolkit.createImage(imagesPath + cardSuitStr.charAt(i) + "_" + cardRankStr.charAt(j) + ".png");
            }
        }

        // Buffers all images for trump suit
        for (int i = 0; i < cardSuitStr.length(); i++) {
            suitImages[i] = toolkit.createImage(imagesPath + cardSuitStr.charAt(i) + ".png");
        }

        // Buffers the the card back image
        backImage = toolkit.createImage(imagesPath + "backside.png");
    }

    /**
     * Returns the image to match card by both rank and suit
     * @param The Card object corresponding to a certain playing card
     * @return The image corresponding to 'card' both by rank and suit
     */
    public Image getImageByCard (Card card) {
        if (card == null) {
            return null;
        }
        
        return cardImages[card.getSuit().ordinal()][card.getRank().ordinal()];
    }

    /**
     *  Returns the image to match card by suit only
     * @param The Card object corresponding to a certain playing card
     * @return The image corresponding to 'card' by just its suit
     */
    public Image getImageBySuit (Card card) {
        return suitImages[card.getSuit().ordinal()];
    }

    /**
     * Returns the image corresponding to suit
     * @param The suit to match
     * @return The image of the suit.
     */
    public Image getImageBySuit (Suit suit) {
        return suitImages[suit.ordinal()];
    }

    /**
     * Returns the image which represents the back of a card.
     * @return The Image object corresponding to an image of the back of a card.
     */
    public Image getBackImage () {
        return backImage;
    }


}

