package GUI;

import Game.Card;
import Game.Card.Rank;
import Game.Card.Suit;
import Game.Hand;
import Game.Pile;
import Main.Main;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import javax.swing.JPanel;

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 *
 * @author A.M. Bosch, S080431
 */
public class FocusCardView extends JPanel implements MouseMotionListener, MouseListener {

    //constructor
    public FocusCardView() {
        super();
        addMouseMotionListener(this);
        addMouseListener(this);
    }
    
    //variables
    private int selected = -1;    //contains index of selected card
    private int numCards = -1;    //contains number of cards in the hand

    private Pile tHand = new Pile(8);
    private Image[] pHand = new Image[8];

    /*
     *  Updates the focusCardView to display a new hand.
     * @param hand, non-null object.
     */
    public void updateFocusCardView(Hand hand){
        CardImage cardImage = CardImage.getCardImage();                         //reference to CardImage
        numCards = hand.getCardCount();                                         //numCards becomes number of cards
        
        tHand.empty();

        for (int i = 0; i < numCards; i++){                                     //Translates hand object to their respective images and stores them in pHand
            tHand.addCard(hand.getCard(i));
            pHand[i] = cardImage.getImageByCard(hand.getCard(i));
        }

        selected = (selected < numCards) ? selected : numCards-1;               // Fixes the case where you don't move mouse and click after removing cards
                                                                                // from the hand (illegal value for selected gives nullptr)
        
        repaint();                                                              //repaint the playerCardPanel
    }

    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        for(int i = 0; i < numCards; i++){                                        //Draws images from pHand, selected cards are raised
            g.drawImage(pHand[i], i*CardImage.XSPACING,(i == selected ? 0 : CardImage.YINDENT), this);
        }
    }


    public void mouseDragged(MouseEvent e) {}

    /*
     * mouseMoved events will be used to select cards. selectCard method is called everytime a mouseMoved event is triggered.
     */
    public void mouseMoved(MouseEvent e) {
        selectCard(e.getX(), e.getY());
    }

    /* selectCard determines which card is selected by the mouseMoved event
     * the result is stored in the global int selected
     * if a different card is selected then before, the panel is repainted
     */
    private void selectCard(int x, int y){
        int oldSelected = selected;

        if (y >= CardImage.YINDENT &&
            y <= CardImage.YINDENT + CardImage.HEIGHT) {            //checks if mouseMoved Event is positioned within the vertical scope
            
            if (x < CardImage.XSPACING * (numCards-1)) {                     //checks if mouseMoved Event is positioned within the horizontal scope
                selected = x / CardImage.XSPACING;
            } else {
                selected = (numCards-1);                                                //outside the horizontal scope:
            }

            if (oldSelected != selected){                                           //check is selected has changes
                repaint();                                                          //repaint if this is the case
            }
        }
    }

    /*
     * mousePressed checks if it can update cardView.selectedCard
     * If so, it changes this variable to the currently selected card.
     */
    public void mousePressed(MouseEvent e) {
        CardView cardView = Main.jFrameMain.cardView;

        if (cardView.getToChooseCard()) {
            cardView.setSelectedCard(tHand.getCard(selected));
            cardView.setToChooseCard(false);
        }
    }

    // Unused methods, must be implemented with the use of MouseListener and MouseMotionListener interfaces.
    public void mouseClicked(MouseEvent e) {}
    public void mouseReleased(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}

}

