/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package Game;

/**
 * Represents a card.
 * @author Perry
 */
public class Card {
    /**
     * Order of cards.
     */
    public static enum Rank {
        TWO, THREE, FOUR, FIVE, SIX, SEVEN, EIGHT, NINE, TEN, JACK, QUEEN, KING, ACE
    }
    /**
     * Order of trumpcards. UNDEF means that the card is lower than SEVENl
     */
    public static enum RankTrump {
        UNDEF, SEVEN, EIGHT, QUEEN, KING, TEN, ACE, NINE, JACK
    }
    /**
     * Order of non-trumpcards. UNDEF means that the card lower than SEVEN.
     */
    public static enum RankNoTrump {
        UNDEF, SEVEN, EIGHT, NINE, JACK, QUEEN, KING, TEN, ACE
    }
    /**
     * Suits a card can have.
     */
    public static enum Suit {
        CLUBS, DIAMONDS, SPADES, HEARTS
    }
    
    private Suit suit;                                                          //suit of the card
    private Rank rank;                                                          //normal rank of the card (used for detecting straights)
    private RankTrump rankTrump;                                                //rank of the card when it is trump
    private RankNoTrump rankNoTrump;                                            //rank of the card when it is not trump
    private int value;                                                          //points the card is normally worth
    private int valueTrump;                                                     //points the card is worth as a trump
    private static int valueArray[] =       {0,0,0,0,2, 3, 4,10,11};            //value of all cards when they are not trump
    private static int valueTrumpArray[] =  {0,0,0,3,4,10,11,14,20};            //value of all cards when they are trump

    /**
     * Should not be called explicitly. Use Card(Rank rank, Suit suit) instead.
     */
    protected Card() {
    }

    /**
     * Normal constructor. Just initializes values. (Card() is only used by desendants)
     * @param rank The rank the card should have as Card.Rank
     * @param suit The suit the card should have as Card.Suit
     */
    public Card(Rank rank, Suit suit) {
        this.rank = rank;
        this.suit = suit;

        switch(rank) {
            case SEVEN:
                rankTrump = RankTrump.SEVEN;
                rankNoTrump = RankNoTrump.SEVEN;
                break;
            case EIGHT:
                rankTrump = RankTrump.EIGHT;
                rankNoTrump = RankNoTrump.EIGHT;
                break;
            case NINE:
                rankTrump = RankTrump.NINE;
                rankNoTrump = RankNoTrump.NINE;
                break;
            case TEN:
                rankTrump = RankTrump.TEN;
                rankNoTrump = RankNoTrump.TEN;
                break;
            case JACK:
                rankTrump = RankTrump.JACK;
                rankNoTrump = RankNoTrump.JACK;
                break;
            case QUEEN:
                rankTrump = RankTrump.QUEEN;
                rankNoTrump = RankNoTrump.QUEEN;
                break;
            case KING:
                rankTrump = RankTrump.KING;
                rankNoTrump = RankNoTrump.KING;
                break;
            case ACE:
                rankTrump = RankTrump.ACE;
                rankNoTrump = RankNoTrump.ACE;
                break;
            default:
                rankTrump = RankTrump.UNDEF;
                rankNoTrump = RankNoTrump.UNDEF;
                break;
        }

        value = valueArray[rankNoTrump.ordinal()];
        valueTrump = valueTrumpArray[rankTrump.ordinal()];
    }

    /**
     * Get this suit.
     * @return suit Suit of card
     */
    public Suit getSuit() {
        return suit;
    }

    /**
     * Get this rank.
     * @return rank Rank of card
     */
    public Rank getRank() {
        return rank;
    }

    public RankTrump getTrumpRank() {
        return rankTrump;
    }

    public RankNoTrump getNoTrumpRank() {
        return rankNoTrump;
    }

    /**
     * Determines whether this card is higher than the card given as argument, or
     * returns true if the argument is null.
     * @param card Card to compare with
     * @param trump The current trump
     * @return true if this is higher than card or card is null. false otherwise. Assumes this is played before card (needed when comparing different suits)
     */
    public boolean higher(Card card, Card.Suit trump){
        if (card == null) {                                                     // if card is no card, then true
            return true;
        }

        if (suit == card.suit) {                                                //Checks whether cards are of the same suit
            if (suit == trump) {                                                //If so, checks if it is trump.
                return rankTrump.ordinal() > card.rankTrump.ordinal();
            } else {
                return (rankNoTrump.ordinal() > card.rankNoTrump.ordinal());
            }
        } else {
            return (card.suit != trump);
        }
    }

    /**
     * Returns value of card
     * @param trump The trumpcolor
     * @return Points this is worth.
     */
    public int value(Card.Suit trump){
        return (suit == trump) ? valueTrump : value;
    }

    @Override public String toString(){
        return rank + " of " + suit;
    }

    public boolean higherOrder(Card card, Card.Suit trump){
        //sort by suit
        if (suit.ordinal() != card.getSuit().ordinal()) {
            return (suit.ordinal() > card.getSuit().ordinal());
        }

        //suits are the same
        return (suit == trump) ?
               (rankTrump.ordinal() > card.getTrumpRank().ordinal()) :
               (rankNoTrump.ordinal() > card.getNoTrumpRank().ordinal());
    }
}
